/**
 * Copyright 2007 by jFramework.org.
 */
package org.jframework.core.util;

import junit.framework.TestCase;

import java.util.*;

import org.apache.log4j.BasicConfigurator;

import junitx.framework.Assert;

/**
 * Dynamic proxy utilities test.
 *
 * @author mgridnev
 * @version 1.0, Feb 1, 2007
 */
public class DynamicProxyUtilsTest extends TestCase
{
    /**
     * Sets up the test run
     */
    protected void setUp() throws Exception
    {
        BasicConfigurator.configure();
    }

    /**
     * Cleans up after test run
     */
    protected void tearDown() throws Exception
    {
        BasicConfigurator.resetConfiguration();
    }

    /**
     * Test dynamic implementation of a basic bean
     */
    public void testImplementBasic()
    {
        final BasicBean bean = DynamicProxyUtils.implement(BasicBean.class);

        final int id = 1;
        bean.setId(id);
        assertEquals(id, bean.getId());

        final String name = "basic bean";
        bean.setName(name);
        assertEquals(name, bean.getName());

        // "is" version
        boolean value = bean.isFlag();
        bean.setFlag(value != value);
        assertEquals(value, bean.isFlag());

        // "get" version
        value = bean.getFlag2();
        bean.setFlag2(value != value);
        assertEquals(value, bean.getFlag2());

        // Not an interface
        try {
            DynamicProxyUtils.implement(BasicBeanImpl.class);
            fail();
        } catch (IllegalArgumentException e) {
        }
    }

    /**
     * Tests implementing a bean that has a superclass
     */
    public void testImplementsWithSuperclass()
    {
        final BasicBean bean = DynamicProxyUtils.implement(BasicBean.class, Bean.class);
        bean.setName("name");

        assertTrue(bean instanceof Bean);

        final BasicBean bean2 = DynamicProxyUtils.implement(BasicBean.class, Bean.class);
        Assert.assertNotEquals(bean, bean2);

        bean2.setName(bean.getName());
        assertEquals(bean, bean2);
        assertEquals(bean.hashCode(), bean2.hashCode());
    }

    /**
     * Tests class semantics are observed
     */
    public void testClassSemantics()
    {
        final Class<BasicBean> inf = BasicBean.class;

        final BasicBean bean = DynamicProxyUtils.implement(inf);
        final BasicBean bean2 = DynamicProxyUtils.implement(inf);

        assertSame(bean.getClass(), bean2.getClass());

        final BasicBean beanWithSuperclass = DynamicProxyUtils.implement(inf, Bean.class);
        Assert.assertNotEquals(bean.getClass(), beanWithSuperclass.getClass());
    }

    /**
     * Tests implementation of a multi-inheritance bean
     */
    public void testImplementInheritied()
    {
        final MultipleInheritanceBean bean = DynamicProxyUtils.implement(MultipleInheritanceBean.class);

        // Basic bean
        final String name = "basic bean";
        bean.setName(name);
        assertEquals(name, bean.getName());

        // Another basic bean
        final String name2 = "another bean";
        bean.setAnotherName(name2);
        assertEquals(name2, bean.getAnotherName());

        // Multi name
        final String name3 = "yet another bean";
        bean.setMultiName(name3);
        assertEquals(name3, bean.getMultiName());
    }

    /**
     * Tests collection property population
     */
    public void testImplementCollectionBean()
    {
        final CollectionBean bean = DynamicProxyUtils.implement(CollectionBean.class);

        assertTrue(bean.getList() instanceof List);
        assertTrue(bean.getSet() instanceof Set);
        assertTrue(bean.getMap() instanceof Map);
    }

    /**
     * Test extraction of unimplemented interfaces
     */
    @SuppressWarnings("unchecked")
    public void testExtractUnimplementedInterfaces()
    {
        // None implemented
        List<Class<?>> interfaces = DynamicProxyUtils.extractUnimplementedInterfaces(new OneTwoImpl(), new OneTwoDecoratorImpl());
        assertEquals(Arrays.asList(OneDecorator.class, TwoDecorator.class), interfaces);

        // One implemented
        interfaces = DynamicProxyUtils.extractUnimplementedInterfaces(new OneDecorator()
        {

            public String getDecoratedOne()
            {
                return null;
            }
        }, new OneTwoDecoratorImpl());
        assertEquals(Arrays.asList(TwoDecorator.class), interfaces);

        // All implemented
        interfaces = DynamicProxyUtils.extractUnimplementedInterfaces(new OneTwoDecoratorImpl(), new OneTwoDecoratorImpl());
        assertEquals(Collections.emptyList(), interfaces);
    }

    /**
     * Tests dynamic decorating
     */
    public void testDecorate()
    {
        final OneTwoImpl original = new OneTwoImpl();
        final OneTwoDecoratorImpl decorator = new OneTwoDecoratorImpl();

        final Object decorated = DynamicProxyUtils.decorate(original, decorator);
        assertTrue(decorated instanceof OneTwoImpl);

        // Call the methods just to make sure
        assertEquals(original.getOne(), ((One) decorated).getOne());
        assertEquals(original.getTwo(), ((Two) decorated).getTwo());

        assertDecorated(decorator, decorated);

        // Double decoration
        final Object doubleDecorated = DynamicProxyUtils.decorate(decorated, decorator);
        assertSame(doubleDecorated, decorated);
    }

    /**
     * Tests decorating a string which is a final class
     */
    public void testDecorateString()
    {
        try {
            DynamicProxyUtils.decorate("string", new OneTwoDecoratorImpl());
            fail();
        } catch (IllegalArgumentException e) {
        }
    }

    /**
     * Tests a basic proxy when decorator is null
     */
    public void testDecorateWithNull()
    {
        try {
            DynamicProxyUtils.decorate(new OneTwoImpl(), null);
            fail();
        } catch (NullPointerException e) {
        }
    }

    /**
     * Tests a duplicate interface decoration
     */
    public void testDuplicateInterface()
    {
        final Object original = DynamicProxyUtils.decorate(new OneTwoImpl(), new DuplicateImpl());
        final Object decorator = DynamicProxyUtils.decorate(new OneTwoDecoratorImpl(), new DuplicateImpl());

        try {
            DynamicProxyUtils.decorate(original, decorator);
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Tests mixing of two objects
     */
    public void testMixed()
    {
        final OneTwoImpl original = new OneTwoImpl();
        final OneTwoDecoratorImpl decorator = new OneTwoDecoratorImpl();

        final Object decorated = DynamicProxyUtils.mix(original, decorator);

        assertTrue(decorated instanceof One);
        assertEquals(original.getOne(), ((One) decorated).getOne());

        assertTrue(decorated instanceof Two);
        assertEquals(original.getTwo(), ((Two) decorated).getTwo());

        assertDecorated(decorator, decorated);

        // Double decoration
        final Object doubleDecorated = DynamicProxyUtils.decorate(decorated, decorator);
        assertSame(doubleDecorated, decorated);
    }


    /**
     * Asserts decorated object
     */
    private static void assertDecorated(final OneTwoDecoratorImpl decorator, final Object decorated)
    {
        assertTrue(decorated instanceof OneDecorator);
        assertEquals(decorator.getDecoratedOne(), ((OneDecorator) decorated).getDecoratedOne());

        assertTrue(decorated instanceof TwoDecorator);
        assertEquals(decorator.getDecoratedTwo(), ((TwoDecorator) decorated).getDecoratedTwo());
    }

    /**
     * A bean containing basic properties
     */
    public static interface BasicBean
    {
        int getId();

        void setId(final int id);

        String getName();

        void setName(final String name);

        boolean isFlag();

        void setFlag(boolean value);

        boolean getFlag2();

        void setFlag2(boolean value);
    }

    /**
     * Another bean with a basic property
     */
    public static interface AnotherBasicBean
    {
        String getAnotherName();

        void setAnotherName(String name);
    }

    /**
     * A multi-inheritance bean
     */
    public static interface MultipleInheritanceBean extends BasicBean, AnotherBasicBean
    {
        String getMultiName();

        void setMultiName(String name);
    }

    /**
     * A bean whose properties are collections
     */
    public static interface CollectionBean
    {
        List getList();

        void setList(List list);

        Set getSet();

        void setSet(Set set);

        Map getMap();

        void setMap(Map map);

        Collection getCollection();

        void setCollection(Collection collection);
    }

    /**
     * A bean containing basic properties
     */
    private static class BasicBeanImpl implements BasicBean
    {
        private int id;
        private String name;
        private boolean flag;
        private boolean flag2;

        public BasicBeanImpl()
        {
            this(0, "");
        }

        public BasicBeanImpl(final int id, final String name)
        {
            this.id = id;
            this.name = name;
        }

        public int getId()
        {
            return id;
        }

        public void setId(final int id)
        {
            this.id = id;
        }

        public String getName()
        {
            return name;
        }

        public void setName(final String name)
        {
            this.name = name;
        }

        public boolean isFlag()
        {
            return flag;
        }

        public void setFlag(final boolean value)
        {
            flag = value;
        }

        public boolean getFlag2()
        {
            return flag2;
        }

        public void setFlag2(final boolean value)
        {
            flag2 = value;
        }
    }

    /**
     * One interface
     */
    static interface One
    {
        String getOne();
    }

    /**
     * Another interface
     */
    static interface Two
    {
        String getTwo();
    }

    /**
     * One-two implementation
     */
    static class OneTwoImpl implements One, Two
    {
        public String getOne()
        {
            return "one";
        }

        public String getTwo()
        {
            return "two";
        }
    }

    /**
     * One decorator
     */
    static interface OneDecorator
    {
        String getDecoratedOne();
    }

    /**
     * Another decorator
     */
    static interface TwoDecorator
    {
        String getDecoratedTwo();
    }

    /**
     * Duplicate interface
     */
    static interface Duplicate
    {
    }

    /**
     * Duplicate implementation
     */
    static class DuplicateImpl implements Duplicate
    {

    }

    /**
     * One-two decorator implementation
     */
    static class OneTwoDecoratorImpl implements OneDecorator, TwoDecorator
    {
        public String getDecoratedOne()
        {
            return "decoratedOne";
        }

        public String getDecoratedTwo()
        {
            return "decoratedTwo";
        }
    }
}
