/**
 * Copyright 2006-2008 by jFramework.org.
 */
package org.jframework.core.util;

import java.text.MessageFormat;
import java.util.ListResourceBundle;
import java.util.Locale;

/**
 * Resource test.
 *
 * @author mgridnev
 * @version 2.0, Jun 1, 2008
 */
public class ResourcesTest extends ResourcesTestCase
{
    /**
     * Context name.
     */
    private static final String CONTEXT = "context";

    /**
     * Simple string key name
     */
    private static final String SIMPLE_KEY = "simpleKey";

    /**
     * Simple string value.
     */
    private static final String SIMPLE_VALUE = "simple value";

    /**
     * Format key name.
     */
    private static final String FORMAT_KEY = "formatKey";

    /**
     * Format string value.
     */
    private static final String FORMAT_VALUE = "{0}";

    /**
     * {@inheritDoc}
     */
    protected Mapping[] createMappings()
    {
        return new Mapping[]{
            new Mapping(CONTEXT, new ListResourceBundle()
            {
                protected Object[][] getContents()
                {
                    return new Object[][]{
                        {SIMPLE_KEY, SIMPLE_VALUE},
                        {FORMAT_KEY, FORMAT_VALUE}
                    };
                }
            })
        };
    }

    /**
     * Tests getting a string from the resource bundle.
     */
    public void testGetString()
    {
        // Valid resource bundle and valid key
        String name = formatName(CONTEXT, SIMPLE_KEY);
        assertEquals(SIMPLE_VALUE, resources.getString(name));

        // Valid resource bundle but invalid key
        final String missingKey = "missing";
        name = formatName(CONTEXT, missingKey);
        assertEquals(name, resources.getString(name)); // returns resource name as default

        // Repeat the above with an explicity default value
        final String defaultString = "default string";
        assertEquals(defaultString, resources.getString(CONTEXT, missingKey, defaultString));

        // Valid context that results in an attempt to load a resource bundle but resource bundle is missing
        String context = "ctx"; // context is valid
        name = formatName(context, SIMPLE_KEY);
        assertEquals(name, resources.getString(name));
        assertTrue(resources.getBundles().containsKey(context)); // null has been cached

        // Invalid context so there's no resource bundle lookup
        context = "invalid context"; // context name has a space, which is an invalid character
        name = formatName(context, SIMPLE_KEY);
        assertEquals(name, resources.getString(name));
        assertFalse(resources.getBundles().containsKey(context)); // nothing should have been cached
    }

    /**
     * Formats resource name.
     *
     * @param context context name
     * @param key     key name
     * @return full resource name
     */
    private static String formatName(String context, String key)
    {
        return context + ":" + key;
    }

    /**
     * Tests string formatting.
     */
    public void testFormatString()
    {
        // Existing string
        String arg = "argument";
        assertEquals(MessageFormat.format(FORMAT_VALUE, arg), resources.formatString(formatName(CONTEXT, FORMAT_KEY), arg));

        // Missing string
        String name = formatName(CONTEXT, "missing");
        assertEquals(name, resources.formatString(name, arg));
    }

    /**
     * Tests loading of locale specific resources.
     */
    public void testLocaleSpecific()
    {
        assertEquals(SIMPLE_VALUE, resources.getString(TestBundle.class.getName() + ":" + SIMPLE_KEY));

        resources.setLocale(new Locale("en", "US", "2"));

        assertEquals(TestBundle_en_US_2.VALUE, resources.getString(TestBundle.class.getName() + ":" + SIMPLE_KEY));
    }

    /**
     * Test bundle in a default locale.
     */
    public static class TestBundle extends ListResourceBundle
    {
        protected Object[][] getContents()
        {
            return new Object[][]{
                {SIMPLE_KEY, SIMPLE_VALUE}
            };
        }
    }

    /**
     * A test bundle for en_US locale.
     */
    public static class TestBundle_en_US_2 extends TestBundle
    {
        public static final String VALUE = "en_US value";

        protected Object[][] getContents()
        {
            return new Object[][]{
                {SIMPLE_KEY, VALUE}
            };
        }
    }
}
